package com.mortals.xhx.module.customer.service.impl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.util.SecurityUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.CustomerSatusEnum;
import com.mortals.xhx.common.code.CustomerSrcEnum;
import com.mortals.xhx.common.code.TrialAcceptEnum;
import com.mortals.xhx.module.customer.dao.CustomerDao;
import com.mortals.xhx.module.customer.model.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.customer.dao.CustomerTrialDao;
import com.mortals.xhx.module.customer.service.CustomerTrialService;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
* CustomerTrialService
* 客户试用申请 service实现
*
* @author zxfei
* @date 2022-06-08
*/
@Service("customerTrialService")
public class CustomerTrialServiceImpl extends AbstractCRUDServiceImpl<CustomerTrialDao, CustomerTrialEntity, Long> implements CustomerTrialService {

    @Autowired
    private CustomerDao customerDao;

    @Override
    protected CustomerTrialEntity findBefore(CustomerTrialEntity entity, PageInfo pageInfo, Context context) throws AppException {
        CustomerTrialQuery query = new CustomerTrialQuery();
        BeanUtils.copyProperties(entity,query);
        if(StringUtils.isNotEmpty(entity.getQuery())){
            StringBuffer condition = new StringBuffer("%");
            condition.append(entity.getQuery()).append("%");
            CustomerTrialQuery condition1 = new CustomerTrialQuery();
            CustomerTrialQuery condition2 = new CustomerTrialQuery();
            condition1.setContactTelphone(condition.toString());
            condition2.setCustName(condition.toString());
            List<CustomerTrialQuery> orConditionList = new ArrayList<>();
            orConditionList.add(condition1);
            orConditionList.add(condition2);
            query.setOrConditionList(orConditionList);
        }
        return query;
    }

    @Override
    protected void saveBefore(CustomerTrialEntity entity, Context context) throws AppException {
        entity.setIsAccept(TrialAcceptEnum.NOT_ACCEPTED.getValue());
        if(entity.getCustomerId()!=null){
            CustomerEntity customerEntity = customerDao.get(entity.getCustomerId());
            if(customerEntity==null){
                throw new AppException("客户信息不存在");
            }
            entity.setJob(customerEntity.getJob());
            entity.setCustName(customerEntity.getCustName());
            entity.setOrganization(customerEntity.getOrganization());
            entity.setContactTelphone(customerEntity.getContactTelphone());
            entity.setSex(customerEntity.getSex());
            entity.setMailbox(customerEntity.getMailbox());
        }
        super.saveBefore(entity, context);
    }

    @Override
    public void doAccept(CustomerTrialEntity entity, Context context) throws AppException {
        CustomerTrialEntity customerTrialEntity = this.dao.get(entity.getId());
        if(customerTrialEntity==null){
            throw new AppException("请求数据不存在");
        }
        if(customerTrialEntity.getIsAccept()!=TrialAcceptEnum.NOT_ACCEPTED.getValue()){
            throw new AppException("请求数据已处理");
        }
        if(entity.getIsAccept()==null){
            throw new AppException("受理意见不能为空");
        }
        if(entity.getIsAccept()== TrialAcceptEnum.AGREE.getValue()){

            if(customerTrialEntity.getCustomerId()!=null){
                CustomerEntity update = new CustomerEntity();
                update.setId(customerTrialEntity.getCustomerId());
                update.setMemberLevel(customerTrialEntity.getMemberLevel());
                update.setUpdateTime(new Date());
                update.setUpdateUserId(this.getContextUserId(context));
                customerDao.update(update);
            }else {
                if(StringUtils.isEmpty(entity.getLoginName())){
                    throw new AppException("登录账号不能为空");
                }
                if(StringUtils.isEmpty(entity.getPassword())){
                    throw new AppException("登录密码不能为空");
                }
                if(StringUtils.isEmpty(entity.getEnterpriseConsultant())){
                    throw new AppException("企业顾问不能为空");
                }
                CustomerEntity customerEntity = new CustomerEntity();
                customerEntity.setJob(customerTrialEntity.getJob());
                customerEntity.setCustName(customerTrialEntity.getCustName());
                customerEntity.setMemberLevel(customerTrialEntity.getMemberLevel());
                customerEntity.setOrganization(customerTrialEntity.getOrganization());
                customerEntity.setContactTelphone(customerTrialEntity.getContactTelphone());
                customerEntity.setSex(customerTrialEntity.getSex());
                customerEntity.setMailbox(customerTrialEntity.getMailbox());
                customerEntity.setCustomerSrc(1l);
                customerEntity.setStatus(CustomerSatusEnum.NORMAL.getValue());
                customerEntity.setEnterpriseConsultant(entity.getEnterpriseConsultant());
                customerEntity.setCreateUserId(this.getContextUserId(context));
                customerEntity.setCreateTime(new Date());
                customerEntity.setLoginName(entity.getLoginName());
                try {
                    customerEntity.setPassword(SecurityUtil.md5DoubleEncoding(entity.getPassword()));
                } catch (Exception e) {
                    throw new AppException("密码转换异常！", e);
                }
                customerDao.insert(customerEntity);
            }

        }
        CustomerTrialEntity update = new CustomerTrialEntity();
        update.setId(customerTrialEntity.getId());
        update.setIsAccept(entity.getIsAccept());
        this.dao.update(update);
    }
}