package com.mortals.xhx.module.customer.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.common.code.WorkDesignStatusEnum;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.customer.dao.CustomerWorkDesignDao;
import com.mortals.xhx.module.customer.model.*;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.customer.service.CustomerWorkDesignService;
import com.mortals.xhx.module.customer.service.CustomerWorkDesignStatService;
import com.mortals.xhx.module.design.model.DesignMasterplateEntity;
import com.mortals.xhx.module.design.service.DesignMasterplateService;
import com.mortals.xhx.module.masterplate.model.MasterplateUseInfoEntity;
import com.mortals.xhx.module.masterplate.service.MasterplateUseInfoService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
* CustomerWorkDesignService
* 客户作品信息 service实现
*
* @author zxfei
* @date 2022-06-14
*/
@Service("customerWorkDesignService")
public class CustomerWorkDesignServiceImpl extends AbstractCRUDServiceImpl<CustomerWorkDesignDao, CustomerWorkDesignEntity, Long> implements CustomerWorkDesignService {

    @Autowired
    private CustomerWorkDesignStatService customerWorkDesignStatService;
    @Autowired
    private DesignMasterplateService designMasterplateService;
    @Autowired
    private MasterplateUseInfoService masterplateUseInfoService;
    @Autowired
    private CustomerService customerService;

    @Override
    protected CustomerWorkDesignEntity findBefore(CustomerWorkDesignEntity params, PageInfo pageInfo, Context context) throws AppException {
        pageInfo.setPrePageResult(-1);

        return super.findBefore(params, pageInfo, context);
    }

    @Override
    protected void saveBefore(CustomerWorkDesignEntity entity, Context context) throws AppException {
        this.validData(entity, context);
        entity.setCustomerId(this.getContextUserId(context));
    }

    @Override
    protected void saveAfter(CustomerWorkDesignEntity entity, Context context) throws AppException {
        CustomerWorkDesignStatEntity cQuery = new CustomerWorkDesignStatEntity();
        cQuery.setCustomerId(entity.getCustomerId());
        List<CustomerWorkDesignStatEntity> cwdList = customerWorkDesignStatService.find(cQuery);
        if(CollectionUtils.isEmpty(cwdList)){
            CustomerWorkDesignStatEntity cwdEntity = new CustomerWorkDesignStatEntity();
            cwdEntity.setCustomerId(entity.getCustomerId());
            if(entity.getDesignType()==1) {
                cwdEntity.setCustomerDesignPictures(1);
                cwdEntity.setCustomerDesignVideos(0);
            }else {
                cwdEntity.setCustomerDesignPictures(0);
                cwdEntity.setCustomerDesignVideos(1);
            }
            cwdEntity.setCreateTime(new Date());
            cwdEntity.setUpdateTime(new Date());
            customerWorkDesignStatService.save(cwdEntity);
        }else {
            CustomerWorkDesignStatEntity cwdEntity = cwdList.get(0);
            CustomerWorkDesignStatQuery updateEntity = new CustomerWorkDesignStatQuery();
            updateEntity.setId(cwdEntity.getId());
            if(entity.getDesignType()==1) {
                updateEntity.setCustomerDesignPicturesIncrement(1);
            }else {
                updateEntity.setCustomerDesignVideosIncrement(1);
            }
            updateEntity.setUpdateTime(new Date());
            customerWorkDesignStatService.update(updateEntity);
        }
        if(entity.getMasterplateId()!=null){
            CustomerEntity customerEntity = customerService.get(entity.getCustomerId());
            MasterplateUseInfoEntity masterplateUseInfoEntity = new MasterplateUseInfoEntity();
            masterplateUseInfoEntity.setMasterplateId(entity.getMasterplateId());
            masterplateUseInfoEntity.setCustomerId(entity.getCustomerId());
            masterplateUseInfoEntity.setCustomerName(customerEntity.getCustName());
            masterplateUseInfoEntity.setCustomerTelphone(customerEntity.getContactTelphone());
            masterplateUseInfoEntity.setCustomerOrganization(customerEntity.getOrganization());
            masterplateUseInfoEntity.setCreateTime(new Date());
            masterplateUseInfoService.save(masterplateUseInfoEntity);
        }
    }

    @Override
    public int release(CustomerWorkDesignEntity entity, Context context,String customerName) throws AppException {
        if(entity.getId()==null){
            throw new AppException("作品信息不能为空");
        }
        if(StringUtils.isEmpty(entity.getMasterplateName())){
            throw new AppException("模板名称不能为空");
        }
        if(StringUtils.isEmpty(entity.getFrontCover())){
            throw new AppException("模板封面图不能为空");
        }
        CustomerWorkDesignEntity workDesign = this.get(entity.getId());
        if(workDesign==null){
            throw new AppException("作品不存在或者已被删除");
        }
        if(workDesign.getWorkDesignStatus()==WorkDesignStatusEnum.RELEASE.getValue()){
            throw new AppException("请勿重复发布");
        }
        DesignMasterplateEntity newMasterplate = new DesignMasterplateEntity();
        newMasterplate.setMasterplateName(entity.getMasterplateName());
        List<DesignMasterplateEntity> masterplates = designMasterplateService.find(newMasterplate);
        if(CollectionUtils.isNotEmpty(masterplates)){
            throw new AppException("模板名称已存在");
        }
        CustomerWorkDesignEntity updateEntity = new CustomerWorkDesignEntity();
        updateEntity.setId(workDesign.getId());
        updateEntity.setWorkDesignStatus(WorkDesignStatusEnum.RELEASE.getValue());
        updateEntity.setUpdateTime(new Date());
        this.update(updateEntity);

        newMasterplate.setCustomerId(workDesign.getCustomerId());
        newMasterplate.setMasterplateName(entity.getMasterplateName());
        newMasterplate.setMasterplateCode(1);
        newMasterplate.setCustomerName(customerName);
        newMasterplate.setMasterplateDesc(entity.getMasterplateName());
        newMasterplate.setMasterplatePath(workDesign.getPreviewUrl());
        newMasterplate.setFontIds(workDesign.getFontIds());
        newMasterplate.setPictureBackgroundIds(workDesign.getPictureBackgroundIds());
        newMasterplate.setPictureIds(workDesign.getPictureIds());
        newMasterplate.setPictureSrcIds(workDesign.getPictureSrcIds());
        newMasterplate.setMasterplateUseNum(0);
        newMasterplate.setPreviewUrl(entity.getFrontCover());
        newMasterplate.setDraft(workDesign.getDraft());
        newMasterplate.setPlateType(entity.getPlateType()==null?1:entity.getPlateType());
        newMasterplate.setDesignType(workDesign.getDesignType());
        newMasterplate.setCreateTime(new Date());
        designMasterplateService.save(newMasterplate,context);
        return 1;
    }

    @Override
    public int cancelRelease(Long designId) throws AppException {
        CustomerWorkDesignEntity updateEntity = new CustomerWorkDesignEntity();
        updateEntity.setId(designId);
        updateEntity.setWorkDesignStatus(WorkDesignStatusEnum.DRAFT.getValue());
        updateEntity.setUpdateTime(new Date());
        return dao.update(updateEntity);
    }

    @Override
    public int remove(Long[] ids, Context context) throws AppException {
        this.removeBefore(ids, context);
        List<Long> idList = new ArrayList<>(Arrays.asList(ids));
        List<CustomerWorkDesignEntity> list = this.find(new CustomerWorkDesignQuery().idList(idList));
        int iRet = this.dao.delete(ids);
        if(CollectionUtils.isNotEmpty(list)){
            list.forEach(e->{
                CustomerWorkDesignStatEntity cQuery = new CustomerWorkDesignStatEntity();
                cQuery.setCustomerId(e.getCustomerId());
                List<CustomerWorkDesignStatEntity> cwdList = customerWorkDesignStatService.find(cQuery);
                CustomerWorkDesignStatEntity cwdEntity = cwdList.get(0);
                CustomerWorkDesignStatQuery updateEntity = new CustomerWorkDesignStatQuery();
                updateEntity.setId(cwdEntity.getId());
                updateEntity.setCustomerDesignPicturesIncrement(-1);
                updateEntity.setUpdateTime(new Date());
                customerWorkDesignStatService.update(updateEntity);
            });
        }
        this.removeAfter(ids, context, iRet);
        return iRet;
    }
}